import numpy as np
import scipy.stats as ss
import pandas as pd
import matplotlib.pyplot as plt
import japanize_matplotlib

in_fn = input('File Name (*.csv) = ') 
d_frame = pd.read_csv(in_fn, encoding='shift_jis')
print(d_frame.keys().values)
nm_X = input('Variable Name_X = ')
nm_Y = input('Variable Name_Y = ')
nm_Z = input('Variable Name_Z = ')
X = d_frame[nm_X].values
Y = d_frame[nm_Y].values
Z = d_frame[nm_Z].values

XYZ = np.vstack((X, Y, Z))
Rs = np.corrcoef(XYZ)      #    Rsは相関行列を表す
Rxy = Rs[0][1]  
Rxz = Rs[0][2]  
Ryz = Rs[1][2]  
#
#     偏相関係数の公式による計算（岡本、2022、p.41）
#
Rxy_z = (Rxy - Rxz * Ryz) / (((1 - Rxz**2) * (1 - Ryz**2)) ** 0.5)
print('\nr_xy.z = ', Rxy_z)
n = len(X)
Axz = np.hstack((np.ones((n,1)), np.reshape(Z,(n,1))))
Bxz = np.linalg.pinv(Axz) @ np.reshape(X, (n,1))
X_Z = X - np.reshape(Axz @ Bxz, n)
Ayz = np.hstack((np.ones((n,1)), np.reshape(Z,(n,1))))
Byz = np.linalg.pinv(Ayz) @ np.reshape(Y, (n,1))
Y_Z = Y - np.reshape(Ayz @ Byz, n)
#
#        変数Ｚのリニアな影響を除いた残差による偏相関係数の計算（定義）
#
Rxy_z_reg = np.corrcoef(X_Z, Y_Z)[0][1]
print('Rxy_z_reg =', Rxy_z_reg)

plt.figure(figsize=(7,7))    #   ４枚の散布図を描く画面

plt.subplot(221)

plt.scatter(X, Y)
plt.title(f'Cor({nm_X},{nm_Y}) = {Rxy:.3f}', fontsize=14)
plt.xlabel(nm_X, fontsize=14)
plt.ylabel(nm_Y, fontsize=14)

plt.subplot(222)

plt.scatter(X_Z, Y_Z)
plt.title(f'P.Cor({nm_X},{nm_Y}|{nm_Z}) = {Rxy_z:.3f}', fontsize=12)
plt.xlabel(f'{nm_X}|{nm_Z}', fontsize=14)
plt.ylabel(f'{nm_Y}|{nm_Z}', fontsize=14)

plt.subplot(223)

plt.scatter(Z, X)
plt.title(f'Cor({nm_Z},{nm_X}) = {Rxz:.3f}', fontsize=14)
plt.xlabel(nm_Z, fontsize=14)
plt.ylabel(nm_X, fontsize=14)

plt.subplot(224)

plt.scatter(Z, Y)
plt.title(f'Cor({nm_Z},{nm_Y}) = {Ryz:.3f}', fontsize=14)
plt.xlabel(nm_Z, fontsize=14)
plt.ylabel(nm_Y, fontsize=14)

plt.tight_layout()
plt.show()



